"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Feature_1 = __importDefault(require("./Feature"));
const Permission_1 = __importDefault(require("./Permission"));
const Warrant_1 = require("../types/Warrant");
const WarrantClient_1 = __importDefault(require("../WarrantClient"));
class Authorization {
    static check(check) {
        return __awaiter(this, void 0, void 0, function* () {
            const accessCheckRequest = {
                warrants: [{
                        objectType: Warrant_1.isWarrantObject(check.object) ? check.object.getObjectType() : check.object.objectType,
                        objectId: Warrant_1.isWarrantObject(check.object) ? check.object.getObjectId() : check.object.objectId,
                        relation: check.relation,
                        subject: Warrant_1.isSubject(check.subject) ? check.subject : { objectType: check.subject.getObjectType(), objectId: check.subject.getObjectId() },
                        context: check.context
                    }],
                consistentRead: check.consistentRead,
                debug: check.debug
            };
            if (WarrantClient_1.default.config.authorizeEndpoint) {
                return this.edgeAuthorize(accessCheckRequest);
            }
            return this.authorize(accessCheckRequest);
        });
    }
    static checkMany(check) {
        return __awaiter(this, void 0, void 0, function* () {
            let warrants = check.warrants.map((warrant) => {
                return {
                    objectType: Warrant_1.isWarrantObject(warrant.object) ? warrant.object.getObjectType() : warrant.object.objectType,
                    objectId: Warrant_1.isWarrantObject(warrant.object) ? warrant.object.getObjectId() : warrant.object.objectId,
                    relation: warrant.relation,
                    subject: Warrant_1.isSubject(warrant.subject) ? warrant.subject : { objectType: warrant.subject.getObjectType(), objectId: warrant.subject.getObjectId() },
                    context: warrant.context
                };
            });
            const accessCheckRequest = {
                op: check.op,
                warrants: warrants,
                consistentRead: check.consistentRead,
                debug: check.debug
            };
            if (WarrantClient_1.default.config.authorizeEndpoint) {
                return this.edgeAuthorize(accessCheckRequest);
            }
            return this.authorize(accessCheckRequest);
        });
    }
    static hasFeature(featureCheck) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.check({
                object: new Feature_1.default(featureCheck.featureId),
                relation: "member",
                subject: featureCheck.subject,
                context: featureCheck.context,
                consistentRead: featureCheck.consistentRead,
                debug: featureCheck.debug
            });
        });
    }
    static hasPermission(permissionCheck) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.check({
                object: new Permission_1.default(permissionCheck.permissionId),
                relation: "member",
                subject: permissionCheck.subject,
                context: permissionCheck.context,
                consistentRead: permissionCheck.consistentRead,
                debug: permissionCheck.debug
            });
        });
    }
    // Private methods
    static authorize(accessCheckRequest) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.post({
                    url: "/v2/authorize",
                    data: accessCheckRequest,
                });
                return response.code === 200;
            }
            catch (e) {
                throw e;
            }
        });
    }
    static edgeAuthorize(warrantCheck) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.post({
                    baseUrl: WarrantClient_1.default.config.authorizeEndpoint,
                    url: "/v2/authorize",
                    data: warrantCheck,
                });
                return response.code === 200;
            }
            catch (e) {
                if (e.code === "cache_not_ready") {
                    return this.authorize(warrantCheck);
                }
                throw e;
            }
        });
    }
}
exports.default = Authorization;
