"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const WarrantClient_1 = __importDefault(require("../WarrantClient"));
const constants_1 = require("../constants");
class Session {
    /**
     * Creates an authorization session in Warrant for the specified userId and returns a session token that can be used to
     * make authorization requests to the Warrant API scoped to the specified userId and tenantId.
     *
     * @param session A session object containing the userId, redirectUrl, and optional tenantId for which the authorization session should be created.
     * @returns A session token that can be passed to any of the Warrant client-side SDKs to allow the SDK to make client-side authorization checks for the specified user.
     */
    static createAuthorizationSession(session) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const sess = yield WarrantClient_1.default.httpClient.post({
                    url: "/v1/sessions",
                    data: Object.assign(Object.assign({}, session), { type: "sess" }),
                });
                return sess.token;
            }
            catch (e) {
                throw e;
            }
        });
    }
    /**
     * Creates a self-service session in Warrant for the user with the specified userId and returns a session token that can be used to
     * make authorized requests to the Warrant API scoped to the specified userId and tenantId.
     *
     * @param session A session object containing the userId, redirectUrl, and optional tenantId for which the self service session should be created.
     * @returns A url pointing to the self-service dashboard that will allow the specified user to make changes to the roles and permissions of users in their tenant.
     */
    static createSelfServiceSession(session, redirectUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const sess = yield WarrantClient_1.default.httpClient.post({
                    url: "/v1/sessions",
                    data: Object.assign(Object.assign({}, session), { type: "ssdash" }),
                });
                return `${constants_1.SELF_SERVICE_DASH_URL_BASE}/${sess.token}?redirectUrl=${redirectUrl}`;
            }
            catch (e) {
                throw e;
            }
        });
    }
}
exports.default = Session;
